# SPDX-FileCopyrightText: Copyright © 2023 Idiap Research Institute <contact@idiap.ch>
#
# SPDX-License-Identifier: GPL-3.0-or-later

"""Tests for Extended TBX11K simplified dataset split 2."""

import pytest


def test_protocol_consistency():
    from ptbench.data.tbx11k_simplified_v2_RS import dataset

    # Default protocol
    subset = dataset.subsets("default")
    assert len(subset) == 3

    assert "train" in subset
    assert len(subset["train"]) == 5241

    assert "validation" in subset
    assert len(subset["validation"]) == 1335

    assert "test" in subset
    assert len(subset["test"]) == 1793
    for s in subset["test"]:
        assert s.key.startswith("images/")

    # Check labels
    for s in subset["train"]:
        assert s.label in [0.0, 1.0]

    for s in subset["validation"]:
        assert s.label in [0.0, 1.0]

    for s in subset["test"]:
        assert s.label in [0.0, 1.0]

    # Cross-validation fold 0-8
    for f in range(9):
        subset = dataset.subsets("fold_" + str(f))
        assert len(subset) == 3

        assert "train" in subset
        assert len(subset["train"]) == 6003
        for s in subset["train"]:
            assert s.key.startswith("images/")

        assert "validation" in subset
        assert len(subset["validation"]) == 1529
        for s in subset["validation"]:
            assert s.key.startswith("images/")

        assert "test" in subset
        assert len(subset["test"]) == 837
        for s in subset["test"]:
            assert s.key.startswith("images/")

        # Check labels
        for s in subset["train"]:
            assert s.label in [0.0, 1.0]

        for s in subset["validation"]:
            assert s.label in [0.0, 1.0]

        for s in subset["test"]:
            assert s.label in [0.0, 1.0]

    # Cross-validation fold 9
    subset = dataset.subsets("fold_9")
    assert len(subset) == 3

    assert "train" in subset
    assert len(subset["train"]) == 6003
    for s in subset["train"]:
        assert s.key.startswith("images/")

    assert "validation" in subset
    assert len(subset["validation"]) == 1530
    for s in subset["validation"]:
        assert s.key.startswith("images/")

    assert "test" in subset
    assert len(subset["test"]) == 836
    for s in subset["test"]:
        assert s.key.startswith("images/")

    # Check labels
    for s in subset["train"]:
        assert s.label in [0.0, 1.0]

    for s in subset["validation"]:
        assert s.label in [0.0, 1.0]

    for s in subset["test"]:
        assert s.label in [0.0, 1.0]


@pytest.mark.skip_if_rc_var_not_set("datadir.tbx11k_simplified")
def test_loading():
    from ptbench.data.tbx11k_simplified_v2_RS import dataset

    def _check_sample(s):
        data = s.data
        assert isinstance(data, dict)
        assert len(data) == 2

        assert "data" in data
        assert len(data["data"]) == 14  # Check radiological signs

        assert "label" in data
        assert data["label"] in [0, 1]  # Check labels

    limit = 30  # use this to limit testing to first images only, else None

    subset = dataset.subsets("default")
    for s in subset["train"][:limit]:
        _check_sample(s)
